import pytest
import numpy as np

from dtgw.common import read_temporal_graph
from dtgw.vertex_signatures import SIGNATURE_PROVIDERS, component_sizes, clustering_coefficients

from helpers import *

def test_degrees():
	ltg = load_tgraph("paper_example_1.edges")
	result = SIGNATURE_PROVIDERS["degrees"]().signatures(ltg)
	np.testing.assert_allclose(
		result[:,ltg.vertex_index.find("r"),:],
		np.array([3, 3, 2, 0, 1, 0])[:,np.newaxis]
	)


def test_neighbor_degrees():
	ltg = load_tgraph("paper_example_1.edges")
	result = SIGNATURE_PROVIDERS["neighbor-degrees"](max_degree=3).signatures(ltg)
	np.testing.assert_allclose(
		result[:,ltg.vertex_index.find("r"),:],
		np.array([
			(3, 2, 2, 1),
			(3, 1, 1, 1),
			(2, 2, 1, 0),
			(0, 0, 0, 0),
			(1, 3, 0, 0),
			(0, 0, 0, 0)
		])
	)


def test_union_degrees():
	ltg = load_tgraph("reaction_1.edges")
	result = SIGNATURE_PROVIDERS["union-degrees"]().signatures(ltg)
	assert result.shape == (4, 16, 1)
	for v, deg in [
		("C1", 2),
		("C5", 3),
		("B1", 1)
	]:
		np.testing.assert_allclose(
			result[:,ltg.vertex_index.find(v),:],
			deg
		)

def test_union_degrees_2():
	ltg = load_tgraph("paper_example_2.edges")
	result = SIGNATURE_PROVIDERS["union-degrees"]().signatures(ltg)
	np.testing.assert_allclose(
		result,
		3
	)


def test_component_sizes():
	ltg = load_tgraph("paper_example_1.edges")
	comp_sizes =  component_sizes(ltg.tgraph)
	for v, size3, size5 in [
		("r", 1, 1),
		("g", 1, 3),
		("b", 2, 3),
		("y", 2, 3),
	]:
		assert comp_sizes[3, ltg.vertex_index.find(v)] == size3
		assert comp_sizes[5, ltg.vertex_index.find(v)] == size5 


def test_avg_neighbor_degrees():
	ltg = load_tgraph("paper_example_2.edges")
	result = SIGNATURE_PROVIDERS["avg-neighbor-degree"]().signatures(ltg)
	for v, expected in [
		("r", ((3, 1), (2, 1.5), (1, 2), (0, 0), (2, 2.5))),
		("g", ((1, 3), (1, 2),   (0, 0), (1, 1), (3, 5/3))),
		("b", ((1, 3), (2, 1.5), (2, 1), (1, 1), (2, 2.5))),
		("y", ((1, 3), (1, 2),   (1, 2), (0, 0), (1, 3))),
	]:
		np.testing.assert_allclose(
			result[:,ltg.vertex_index.find(v)],
			expected
		)


def test_density():
	ltg = load_tgraph("paper_example_1.edges")
	result = SIGNATURE_PROVIDERS["degrees-density"]().signatures(ltg)
	for v, expected in [
		("r", ((3, 4/6), (3, 3/6), (2, 3/6), (0, 0), (1, 3/6), (0, 0))),
		("b", ((1, 4/6), (1, 3/6), (2, 3/6), (1, 1), (1, 3/6), (1, 2/3))),
	]:
		np.testing.assert_allclose(
			result[:,ltg.vertex_index.find(v)],
			expected
		)


def test_clustering_coefficient():
	ltg = load_tgraph("paper_example_1.edges")
	result = clustering_coefficients(ltg.tgraph)
	for v, expected in [
		("r", (1/3, 0, 0, 0, 0, 0)),
		("b", (  0, 0, 0, 0, 0, 0)),
		("g", (  1, 0, 0, 0, 0, 0)),
		("y", (  1, 0, 0, 0, 0, 0)),
	]:
		np.testing.assert_allclose(
			result[:, ltg.vertex_index.find(v)],
			expected
		)
